package ueb094camera;

import java.util.ArrayList;

import processing.core.*;
import saito.objloader.*;

@SuppressWarnings("serial")
public class Ueb094Camera extends PApplet {

	private PImage groundTexture;
	private OBJModel house;

	private ArrayList<CameraPoint> points = new ArrayList<CameraPoint>();
	private int activePoint = 0;
	
	private boolean makeMovie = false;
	private long frame = 0;

	public void setup() {
		size(800, 600, OPENGL);
		// load ground texture
		groundTexture = loadImage("ground.jpg");
		// load house model
		house = new OBJModel(this, "CasaSimples.obj", "relative", POLYGON);

		// turning on the debug output (it's all the stuff that spews out in the
		// black box down the bottom)
		// house.enableDebug();
		// house.translateToCenter();
		house.scale(15);
		// house.printModelInfo();

		frameRate(50);

		noStroke();

		initializeCameraRide();
	}

	public void draw() {
		translate(width / 2, height, -500);

		background(color(0, 125, 175));
		// lights();

		drawGround();

		drawHouses(100, 350, 0);
		drawHouses(350, 350, 0);

		drawHouses(750, 70, PI / 2 * 3);
		drawHouses(850, 350, 0);

		drawHouses(200, 800, PI / 2 * 2);
		drawHouses(500, 800, PI / 2 * 2);
		drawHouses(800, 800, PI / 2 * 2);

		moveCamera();
	}

	private void drawHouses(int x, int z, float angle) {
		pushMatrix();
		// breite: 1000 -> -500 vom zentrum zum linken rand
		// Haus 120px breit (60px), ca. -40 im Boden
		translate(-500 + x + 60, -40, -500 + z + 60);
		rotateY(angle);
		house.draw();

		popMatrix();

	}

	private void drawGround() {

		beginShape(QUADS);
		texture(groundTexture);
		vertex(-500, 0, 500, 0, groundTexture.height); // top-left
		vertex(500, 0, 500, groundTexture.width, groundTexture.height); // top-right
		vertex(500, 0, -500, groundTexture.width, 0); // bottom-right
		vertex(-500, 0, -500, 0, 0); // bottom-left
		endShape(CLOSE);
	}

	private void moveCamera() {

		if (activePoint + 1 >= points.size()) {

			// Stoppe sinlosses neuzeichnen
			noLoop();

			System.out.println("Fertig :)");

			return;
		}
		
		CameraPoint currentPoint = points.get(activePoint);
		CameraPoint nextPoint = points.get(activePoint + 1);
		CameraPoint calculatedPoint = new CameraPoint(new Coordinate(),
				new Coordinate());

		boolean movePosition = movePoint(currentPoint.getEye(),
				nextPoint.getEye(), calculatedPoint.getEye());
		boolean moveTarget = movePoint(currentPoint.getTarget(),
				nextPoint.getTarget(), calculatedPoint.getTarget());

		if (!movePosition && !moveTarget) {
			activePoint++;
		}

		camera(calculatedPoint.getEye().getX(),
				calculatedPoint.getEye().getY(), calculatedPoint.getEye()
						.getZ(), calculatedPoint.getTarget().getX(),
				calculatedPoint.getTarget().getY(), calculatedPoint.getTarget()
						.getZ(), 0, 1, 0);

		points.set(activePoint, calculatedPoint);

		System.out.println("Camera Eye: " + calculatedPoint.getEye());
		System.out.println("Camera Target: " + calculatedPoint.getTarget());

		if (makeMovie) {
			saveFrame("Frames/Frame" + String.format("%04d", frame++) + ".png");
		}
	}

	private boolean movePoint(Coordinate source, Coordinate target,
			Coordinate newCoordinate) {

		if (source.getX() == target.getX() && source.getY() == target.getY()
				&& source.getZ() == target.getZ()) {

			newCoordinate.setX(source.getX());
			newCoordinate.setY(source.getY());
			newCoordinate.setZ(source.getZ());

			return false;
		}

		float max = max(new float[] { abs(source.getX() - target.getX()),
				abs(source.getY() - target.getY()),
				abs(source.getZ() - target.getZ()) });

		// Speed
		max = max / 4;

		newCoordinate.setX(source.getX()
				+ ((target.getX() - source.getX()) / max));
		newCoordinate.setY(source.getY()
				+ ((target.getY() - source.getY()) / max));
		newCoordinate.setZ(source.getZ()
				+ ((target.getZ() - source.getZ()) / max));

		return true;
	}

	private void initializeCameraRide() {
		// X (breite) | Z (Hhe ber Grund) | Y (hhe)
		points.add(new CameraPoint(new Coordinate(-300 , height - 700, -1200), new Coordinate(800, height, -200)));
		points.add(new CameraPoint(new Coordinate(200, height - 500, -200), new Coordinate(600, height, -800)));
		points.add(new CameraPoint(new Coordinate(1200, height - 400, 200), new Coordinate(-800, height, -1500)));
//		points.add(new CameraPoint(new Coordinate(200, height - 200, -3000), new Coordinate(200, height - 200f, -400)));
	}

}
