/****************************************************************
* Modul:   TA.BA_MMT.HS1301 - Multimedia Technik (MMT)
* Subject: Processing - Uebung 9.4 - Kamerafahrt
* Author:  Felix Rohrer
* Desc:    3D mit Processing, Kamerafahrt durch die Szene
****************************************************************/

float radX = 2.66;
float radY = 0.77;
float radZ = 0;
int z = 0;
int zoom = 75;

float angle = 0;

PImage eins;
PImage zwei;
PImage drei;
PImage vier;
PImage fuenf;
PImage sechs;

int imgWidth;
int imgHeight;

// ----- setup() ------------------------------------------------
void setup() {
  size(600, 600, OPENGL);
  frameRate(50);
  fill(220);
  strokeWeight(3);
  
  eins  = loadImage("1.gif");
  zwei  = loadImage("2.gif");
  drei  = loadImage("3.gif");
  vier  = loadImage("4.gif");
  fuenf = loadImage("5.gif");
  sechs = loadImage("6.gif");
  
  // alle Bilder sind gleich gross
  imgWidth  = eins.width;
  imgHeight = eins.height;
}


// ----- draw() -------------------------------------------------
void draw()
{
  background(100);
  // zentrieren
  translate(width/2, height/2); 
  
  float cameraX = sin(angle) * 700;
  float cameraZ = cos(angle) * 700;
  
  camera(cameraX, 0, cameraZ, // eye (x,y,z)
         0, 0, 0,             // center of the scene (x,y,z)
         0, 1, 0);            // camera up (x,y,z)
  angle += 0.005;
  
  ambientLight(50, 50, 50);
  pointLight(255,255,255, -300, -400, 0);
  
  //radX += 0.005;
  //radY -= 0.007;
  //radZ += 0.007;
  
  drawWuerfel(100, 0, 0, 0);
  drawWuerfel(50, 0, 100, 200);
  
  drawWuerfel(30, -300, -150, -50);
  
  drawWuerfel(30,    0, 190, -150);
  drawWuerfel(30, -100, 190, -150);
  drawWuerfel(30,  100, 190, -150);
  
  drawFloor();
}


// ----- mouseDragged() -----------------------------------------
void mouseDragged() {
  radX = (float)mouseY/(width/2)*PI; 
  radY = (float)mouseX/(height/2)*PI; 
  //radZ = (float)mouseX/(height/2)*PI;
}


// ----- keyPressed() -------------------------------------------
void keyPressed() {
  if (key == 's') {
    save("printscreen.jpg");
  }
}


// ----- drawFloor() --------------------------------------------
void drawFloor() {
  pushMatrix();
  
  scale(300);
  translate(0,-0.2, 0);
  
  fill(150);
  beginShape(QUADS);
 
  // x --, y | , z /
  vertex(-1, 1, 1); 
  vertex( 1, 1, 1); 
  vertex( 1,  1, -1); 
  vertex(-1,  1, -1); 
  endShape();
 
  popMatrix();
}


// ----- drawWuerfel() ------------------------------------------
void drawWuerfel(int zoom, int x, int y, int z)
{
  pushMatrix();
  
  translate(x, y, z);
  
  // skalierung
  scale(zoom);
  
  // Würfel rotieren
  rotateX(radX);  //rotiert das objekt 
  rotateY(radY);  //rotiert das objekt 
  rotateZ(radZ);  //rotiert das objekt 
   
  // 1. Seite | Front
  beginShape(QUADS);
  tint(255,0,0);
  texture(eins);
  vertex(-1, -1, 1, 0, 0); // top-left
  vertex( 1, -1, 1, imgWidth, 0); // top-right
  vertex( 1,  1, 1, imgWidth, imgHeight); // bottom-right
  vertex(-1,  1, 1, 0, imgHeight); // bottom-left
  endShape(CLOSE);
  
  // 2. Seite | Rechts
  beginShape(QUADS);
  tint(0,255,0);
  texture(fuenf);
  vertex( 1, -1,  1, 0, 0); // top-right
  vertex( 1, -1, -1, imgWidth, 0);
  vertex( 1,  1, -1, imgWidth, imgHeight);
  vertex( 1,  1,  1, 0, imgHeight); // bottom-right
  endShape(CLOSE);
  
  // 3. Seite | Links
  beginShape(QUADS);
  tint(0,0,255);
  texture(zwei);
  vertex(-1, -1,  1, 0, 0); // top-left
  vertex(-1, -1, -1, imgWidth, 0);
  vertex(-1,  1, -1, imgWidth, imgHeight);
  vertex(-1,  1,  1, 0, imgHeight); // bottom-left
  endShape(CLOSE);
  
  // 4. Seite | Hinten
  beginShape(QUADS);
  tint(255,255,0);
  texture(sechs);
  vertex(-1, -1, -1, 0, 0); // hinten-top-left
  vertex( 1, -1, -1, imgWidth, 0);
  vertex( 1,  1, -1, imgWidth, imgHeight);
  vertex(-1,  1, -1, 0, imgHeight); // hinten-bottom-left  
  endShape(CLOSE);
    
  // 5. Seite | Oben
  beginShape(QUADS);
  tint(255,0,255);
  texture(drei);
  vertex(-1, -1,  1, 0, 0); // top-left
  vertex( 1, -1,  1, imgWidth, 0); // top-right
  vertex( 1, -1, -1, imgWidth, imgHeight);
  vertex(-1, -1, -1, 0, imgHeight);
  endShape(CLOSE);
    
  // 6. Seite | Unten
  beginShape(QUADS);
  tint(0,255,255);
  texture(vier);
  vertex(-1,  1,  1, 0, 0); // bottom-left
  vertex( 1,  1,  1, imgWidth, 0); // bottom-right
  vertex( 1,  1, -1, imgWidth, imgHeight);
  vertex(-1,  1, -1, 0, imgHeight);
  endShape(CLOSE);
  
  popMatrix();
}

